/*
 * Copyright © 2014-2017 Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 */

#ifndef _INTEL_DEVICE_INFO_H_
#define _INTEL_DEVICE_INFO_H_

#include <uapi/drm/i915_drm.h>

#include "intel_step.h"

#include "display/intel_display.h"

#include "gt/intel_engine_types.h"
#include "gt/intel_context_types.h"
#include "gt/intel_sseu.h"

#include "gem/i915_gem_object_types.h"

struct drm_printer;
struct drm_i915_private;
struct intel_gt_definition;

/* Keep in gen based order, and chronological order within a gen */
enum intel_platform {
	INTEL_PLATFORM_UNINITIALIZED = 0,
	/* gen2 */
	INTEL_I830,
	INTEL_I845G,
	INTEL_I85X,
	INTEL_I865G,
	/* gen3 */
	INTEL_I915G,
	INTEL_I915GM,
	INTEL_I945G,
	INTEL_I945GM,
	INTEL_G33,
	INTEL_PINEVIEW,
	/* gen4 */
	INTEL_I965G,
	INTEL_I965GM,
	INTEL_G45,
	INTEL_GM45,
	/* gen5 */
	INTEL_IRONLAKE,
	/* gen6 */
	INTEL_SANDYBRIDGE,
	/* gen7 */
	INTEL_IVYBRIDGE,
	INTEL_VALLEYVIEW,
	INTEL_HASWELL,
	/* gen8 */
	INTEL_BROADWELL,
	INTEL_CHERRYVIEW,
	/* gen9 */
	INTEL_SKYLAKE,
	INTEL_BROXTON,
	INTEL_KABYLAKE,
	INTEL_GEMINILAKE,
	INTEL_COFFEELAKE,
	INTEL_COMETLAKE,
	/* gen11 */
	INTEL_ICELAKE,
	INTEL_ELKHARTLAKE,
	INTEL_JASPERLAKE,
	/* gen12 */
	INTEL_TIGERLAKE,
	INTEL_ROCKETLAKE,
	INTEL_DG1,
	INTEL_ALDERLAKE_S,
	INTEL_ALDERLAKE_P,
	INTEL_XEHPSDV,
	INTEL_DG2,
	INTEL_PONTEVECCHIO,
	INTEL_METEORLAKE,
	INTEL_MAX_PLATFORMS
};

/*
 * Subplatform bits share the same namespace per parent platform. In other words
 * it is fine for the same bit to be used on multiple parent platforms.
 */

#define INTEL_SUBPLATFORM_BITS (3)
#define INTEL_SUBPLATFORM_MASK (BIT(INTEL_SUBPLATFORM_BITS) - 1)

/* HSW/BDW/SKL/KBL/CFL */
#define INTEL_SUBPLATFORM_ULT	(0)
#define INTEL_SUBPLATFORM_ULX	(1)

/* ICL */
#define INTEL_SUBPLATFORM_PORTF	(0)

/* TGL */
#define INTEL_SUBPLATFORM_UY	(0)

/* DG2 */
#define INTEL_SUBPLATFORM_G10	0
#define INTEL_SUBPLATFORM_G11	1
#define INTEL_SUBPLATFORM_G12	2

/* ADL */
#define INTEL_SUBPLATFORM_RPL	0

/* ADL-P */
/*
 * As #define INTEL_SUBPLATFORM_RPL 0 will apply
 * here too, SUBPLATFORM_N will have different
 * bit set
 */
#define INTEL_SUBPLATFORM_N    1

/* MTL */
#define INTEL_SUBPLATFORM_M	0
#define INTEL_SUBPLATFORM_P	1

#define DEV_INFO_FOR_EACH_FLAG(func) \
	func(is_lp); \
	func(is_dgfx); \
	/* Keep has_* in alphabetical order */ \
	func(has_64k_pages); \
	func(has_access_counter); \
	func(has_asid_tlb_invalidation); \
	func(has_cache_clos); \
	func(has_gmd_id); \
	func(has_3d_pipeline); \
	func(has_4tile); \
	func(has_eu_stall_sampling); \
	func(has_flat_ccs); \
	func(has_full_ps64); \
	func(has_global_mocs); \
	func(has_gt_error_vectors); \
	func(has_gt_uc); \
	func(has_guc_deprivilege); \
	func(has_guc_programmable_mocs); \
	func(has_heci_pxp); \
	func(has_heci_gscfi); \
	func(has_iaf); \
	func(has_iov_memirq); \
	func(has_l3_ccs_read); \
	func(has_l3_dpf); \
	func(has_link_copy_engines); \
	func(has_llc); \
	func(has_lmtt_lvl2); \
	func(has_media_ratio_mode); \
	func(has_mem_sparing); \
	func(has_mslice_steering); \
	func(has_oa_bpc_reporting); \
	func(has_oa_buf_128m); \
	func(has_oa_mmio_trigger); \
	func(has_oa_slice_contrib_limits); \
	func(has_oac); \
	func(has_oam); \
	func(has_one_eu_per_fuse_bit); \
	func(has_pooled_eu); \
	func(has_pxp); \
	func(has_recoverable_page_fault); \
	func(has_remote_tiles); \
	func(has_runtime_pm); \
	func(has_selective_tlb_invalidation); \
	func(has_semaphore_xehpsdv); \
	func(has_slim_vdbox); \
	func(has_snoop); \
	func(has_sriov); \
	func(has_um_queues); \
	func(tuning_thread_rr_after_dep); \
	func(has_csc_uid);	\
	func(has_lmem_max_bandwidth);	\
	func(unfenced_needs_alignment); \
	func(hws_needs_physical); \
	func(oam_uses_vdbox0_channel); \
	func(needs_driver_flr);

#define DEV_INFO_DISPLAY_FOR_EACH_FLAG(func) \
	/* Keep in alphabetical order */ \
	func(has_cdclk_crawl); \
	func(has_dmc); \
	func(has_ddi); \
	func(has_dsb); \
	func(has_dsc); \
	func(has_fpga_dbg); \
	func(has_hdcp); \
	func(has_hti); \
	func(has_modular_fia); \
	func(has_psr); \
	func(has_psr_hw_tracking);

struct ip_version {
	u8 ver;
	u8 rel;
	u8 step;
};

struct intel_device_info {
	struct ip_version graphics;
	struct ip_version media;

	intel_engine_mask_t platform_engine_mask; /* Engines supported by the HW */

	enum intel_platform platform;

	unsigned int dma_mask_size; /* available DMA address bits */

	unsigned int ppgtt_size; /* log2, e.g. 31/32/48 bits */

	unsigned int ppgtt_msb; /* Virtual Addresss msb supported by the HW */

	unsigned int page_sizes; /* page sizes supported by the HW */

	u32 memory_regions; /* regions supported by the HW */

	const struct intel_gt_definition *extra_gt_list;

	u8 gt; /* GT number, 0 if undefined */

#define DEFINE_FLAG(name) u8 name:1
	DEV_INFO_FOR_EACH_FLAG(DEFINE_FLAG);
#undef DEFINE_FLAG

#if IS_ENABLED(CPTCFG_DRM_I915_DISPLAY)
	struct {
		u8 ver;
		u8 rel;

		u8 pipe_mask;
		u8 cpu_transcoder_mask;
		u8 fbc_mask;
		u8 abox_mask;

		struct {
			u16 size; /* in blocks */
			u8 slice_mask;
		} dbuf;

#define DEFINE_FLAG(name) u8 name:1
		DEV_INFO_DISPLAY_FOR_EACH_FLAG(DEFINE_FLAG);
#undef DEFINE_FLAG

		/* Global register offset for the display engine */
		u32 mmio_offset;

		/* Register offsets for the various display pipes and transcoders */
		u32 pipe_offsets[I915_MAX_TRANSCODERS];
		u32 trans_offsets[I915_MAX_TRANSCODERS];
		u32 cursor_offsets[I915_MAX_PIPES];

		struct {
			u32 degamma_lut_size;
			u32 gamma_lut_size;
			u32 degamma_lut_tests;
			u32 gamma_lut_tests;
		} color;
	} display;
#endif

	unsigned int cachelevel_to_pat[I915_MAX_CACHE_LEVEL];
};

struct intel_runtime_info {
	/*
	 * Platform mask is used for optimizing or-ed IS_PLATFORM calls into
	 * into single runtime conditionals, and also to provide groundwork
	 * for future per platform, or per SKU build optimizations.
	 *
	 * Array can be extended when necessary if the corresponding
	 * BUILD_BUG_ON is hit.
	 */
	u32 platform_mask[2];

	/*
	 * On modern platforms, the architecture major.minor version numbers
	 * and stepping are read directly from the hardware rather than derived
	 * from the PCI device and revision ID's.
	 *
	 * Note that the hardware gives us a single "graphics" number that
	 * should represent render, compute, and copy behavior.
	 */
	struct ip_version graphics;
	struct ip_version media;
	struct ip_version display;

	u16 device_id;

	u64 uid; /* device uid, used for generating uuid */

#if IS_ENABLED(CPTCFG_DRM_I915_DISPLAY)
	u8 num_sprites[I915_MAX_PIPES];
	u8 num_scalers[I915_MAX_PIPES];

#endif
	u32 rawclk_freq;

	struct intel_step_info step;
};

struct intel_driver_caps {
	unsigned int scheduler;
	bool has_logical_contexts:1;
};

const char *intel_platform_name(enum intel_platform platform);

void intel_device_info_subplatform_init(struct drm_i915_private *dev_priv);
#if IS_ENABLED(CPTCFG_DRM_I915_DISPLAY)
void intel_device_info_runtime_init(struct drm_i915_private *dev_priv);

#else
static inline void intel_device_info_runtime_init(struct drm_i915_private *dev_priv) { return; }
#endif
void intel_device_info_print_static(const struct intel_device_info *info,
				    struct drm_printer *p);
void intel_device_info_print_runtime(const struct intel_runtime_info *info,
				     struct drm_printer *p);

void intel_driver_caps_print(const struct intel_driver_caps *caps,
			     struct drm_printer *p);

#endif
